"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.Attribute = void 0;
var compound_1 = require("./compound");
var dimension_1 = require("./dimension");
/**
 * Contains all software used for the calculation of attributes.
 */
var Attribute;
(function (Attribute_1) {
    ;
    /**
     * Contains information for an attribute.
     */
    var Attribute = /** @class */ (function (_super) {
        __extends(Attribute, _super);
        /**
         * Define a new attribute.
         * @param exponents The exponents of the attributes that make up this type
         * @param latex A valid LaTeX equation representing this attributes
         * @param base Determines the base dimension measured by this type (overrides `exponents`)
         */
        function Attribute(exponents, latex, base) {
            var _a;
            if (latex === void 0) { latex = ''; }
            var _this = this;
            var _b;
            var dimExpTemp = base ? (_a = {}, _a[base] = 1, _a) : exponents;
            _this = _super.call(this, dimExpTemp, function (t) { return Table[t]().latex; }) || this;
            _this.latex = latex;
            if (base) {
                _this.dimension = new dimension_1.Dimension.Dimension(dimExpTemp);
            }
            else {
                _this.dimension = dimension_1.Dimension.None;
                for (var m in exponents) {
                    var exponent = (_b = exponents[m]) !== null && _b !== void 0 ? _b : 0;
                    _this.dimension = _this.dimension.mult(Table[m]().dimension, exponent);
                }
            }
            return _this;
        }
        Attribute.prototype.mult = function (other, exponent) {
            return new Attribute(_super.prototype.combine.call(this, other, exponent));
        };
        /**
         * Return the common attribute name for this type, or undefined if none exist.
         * @returns The common attribute name, if one exists
         */
        Attribute.prototype.getName = function () {
            for (var m in Table) {
                if (this.dimension.is(Table[m]().dimension)) {
                    return m;
                }
            }
            return undefined;
        };
        /**
         * Determine the equivalent attribute type, else return this object.
         * @returns The equivalent attribute type, if one exists
         * @example
         * `mass` * `velocity^2` => `energy`
         * `time` * `time` => `time^2` // no equivalent
         */
        Attribute.prototype.simplify = function () {
            var _a;
            var name = this.getName();
            if (name) {
                return new Attribute((_a = {}, _a[name] = 1, _a), Table[name]().latex);
            }
            else {
                return this;
            }
        };
        Attribute.prototype.toString = function () {
            return (this.latex || _super.prototype.toString.call(this));
        };
        return Attribute;
    }(compound_1.Compound.Compound));
    Attribute_1.Attribute = Attribute;
    /**
     * Contains information for common types of attributes.
     */
    var Table = {
        time: function () { return new Attribute({}, 't', 'time'); },
        length: function () { return new Attribute({}, 'x', 'length'); },
        mass: function () { return new Attribute({}, 'm', 'mass'); },
        current: function () { return new Attribute({}, 'I', 'current'); },
        temperature: function () { return new Attribute({}, 'T', 'temperature'); },
        substance: function () { return new Attribute({}, 'n', 'substance'); },
        intensity: function () { return new Attribute({}, 'I_{V}', 'intensity'); },
        area: function () { return new Attribute({ length: 2 }, 'A'); },
        volume: function () { return new Attribute({ length: 3 }, 'V'); },
        velocity: function () { return new Attribute({ length: 1, time: -1 }, 'v'); },
        acceleration: function () { return new Attribute({ length: 1, time: -2 }, 'a'); },
        force: function () { return new Attribute({ mass: 1, acceleration: 1 }, 'F'); },
        energy: function () { return new Attribute({ force: 1, length: 1 }, 'E'); }, // Same as mv^2, mgh, ...
        power: function () { return new Attribute({ energy: 1, time: -1 }, 'P'); },
    };
    /**
     * Represents a dimensionless attribute.
     */
    Attribute_1.None = new Attribute({});
})(Attribute || (exports.Attribute = Attribute = {}));
