"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.D = exports.A = exports.U = exports.Q = void 0;
var attribute_1 = require("./attribute");
var dimension_1 = require("./dimension");
var quantity_1 = require("./quantity");
var unit_1 = require("./unit");
/**
 * Shorthand for creating a new measured quantity.
 * @param value The numerical value of the measurement
 * @param unit The physical units of the measurement
 * @returns A new quantity object
 */
function Q(value, unit) {
    return new quantity_1.Quantity.Quantity(value, unit);
}
exports.Q = Q;
/**
 * Shorthand for creating a new unit of measurement.
 * @param exponents Exponents on each of the individual units
 * @returns A new unit object
 */
function U(exponents) {
    var _a;
    return new unit_1.Unit.Unit((typeof exponents === 'object') ? exponents : (_a = {}, _a[exponents] = 1, _a));
}
exports.U = U;
/**
 * Shorthand for creating a new attribute.
 * @param exponents Exponents that make up this attribute
 * @returns A new attribute object
 */
function A(exponents) {
    var _a;
    return new attribute_1.Attribute.Attribute((typeof exponents === 'object') ? exponents : (_a = {}, _a[exponents] = 1, _a));
}
exports.A = A;
/**
 * Shorthand for creating a set of physical base dimensions.
 * @param exponents Exponents of each of the dimensions
 * @returns A new dimension object
 */
function D(exponents) {
    var _a;
    return new dimension_1.Dimension.Dimension((typeof exponents === 'object') ? exponents : (_a = {}, _a[exponents] = 1, _a));
}
exports.D = D;
