import { Compound } from './compound';
/**
 * Contains all software used for the calculation of physical base dimensions.
 */
export declare namespace Dimension {
    /**
     * A list of common names for each of the physical base dimensions.
     */
    type Name = 'time' | 'length' | 'mass' | 'current' | 'temperature' | 'substance' | 'intensity';
    /**
     * Is an object containing keys of dimensions and values of nonzero exponents.
     */
    interface Exponents extends Compound.Exponents<Name> {
    }
    /**
     * Defines the class for physical base dimensions.
     */
    class Dimension extends Compound.Compound<Name, Dimension> {
        /**
         * Determine if this dimension is a "simple" base dimension.
         */
        readonly isBase: boolean;
        constructor(exponents: Exponents);
        mult(other: Dimension, exponent: number): Dimension;
    }
    /**
     * Represents a dimensionless value.
     */
    const None: Dimension;
}
