import { Attribute } from './attribute';
import { Compound } from './compound';
/**
 * Contains all software used for the calculation of units of measurement.
 */
export declare namespace Unit {
    /**
     * Contains a list of all physical units of measurement.
     */
    type Name = 'nanosecond' | 'millisecond' | 'second' | 'minute' | 'hour' | 'day' | 'week' | 'month' | 'year' | 'nanometer' | 'micrometer' | 'micron' | 'millimeter' | 'centimeter' | 'meter' | 'kilometer' | 'inch' | 'foot' | 'yard' | 'mile' | 'milligram' | 'gram' | 'kilogram' | 'tonne' | 'ounce' | 'pound_mass' | 'slug' | 'stone' | 'shortton' | 'milliampere' | 'ampere' | 'Kelvin' | 'Rankine' | 'Celsius_delta' | 'Fahrenheight_delta' | 'mole' | 'candela' | 'lumen' | 'Newton' | 'kiloNewton' | 'pound_force' | 'Joule' | 'Watt';
    /**
     * Is an object containing keys of units and values of nonzero exponents.
     */
    interface Exponents extends Compound.Exponents<Name> {
    }
    /**
     * Defines the class for units of measurement for physical quantities.
     */
    class Unit extends Compound.Compound<Name, Unit> {
        /**
         * The scale of this unit relative to the base unit.
         */
        readonly scale: number;
        /**
         * The physical attribute measured by this unit.
         */
        readonly attribute: Attribute.Attribute;
        constructor(exponents: Exponents, attrOverride?: Attribute.Attribute);
        mult(other: Unit, exponent: number): Unit;
    }
    /**
     * Represents a unitless value.
     */
    const None: Unit;
}
